@@ -2,9 +2,6 @@
 package PRANG::Cookbook;
 
 use Moose::Role;
-use PRANG::Graph;
-
-sub xmlns { }
 
 BEGIN { with 'PRANG::Graph', 'PRANG::Cookbook::Node'; };
 
@@ -28,9 +25,6 @@ these form the PRANG manual.  However, the cookbook is designed to
 give quick examples to get up to speed with what you need to know to
 effectively use PRANG.
 
-The manual pages for the cookbook do not contain the content; see the
-actual source for the examples.
-
 =head1 RECIPES
 
 =head2 Basic PRANG
@@ -50,9 +44,9 @@ some basic validation.
 
 =head2 Intermediate PRANG
 
-This recipe series (ie, L<PRANG::Cookbook>) gives you a good overview
-of some of advanced of PRANG's capabilites. Showing how to do lists of
-nodes and then lists of any of a number of different nodes.
+This recipe series gives you a good overview of some of advanced of PRANG's
+capabilites. Showing how to do lists of nodes and then lists of any of a number
+of different nodes.
 
 =over 4
 
@@ -73,7 +67,7 @@ will be released shortly after the inaugral PRANG release.
 
 =head1 AUTHOR
 
-Andrew Chilton, E<lt>andy@catalyst dot net dot nzE<gt>
+Andrew Chilton, E<lt>andy@catalyst dot net dot nz<gt>
 
 Edited by Sam Vilain, L<samv@cpan.org>
 
