@@ -234,13 +234,11 @@ method show_node {
 				$_->name."='".$_->value."'"
 			} $node->attributes);
 		}
-		my @nodes = grep {
-			!( $_->isa("XML::LibXML::Comment") or
-				$_->isa("XML::LibXML::Text") and $_->data =~ /\A\s+\Z/
-			) } $node->childNodes;
+		my @nodes = grep { !$_->isa("XML::LibXML::Comment") }
+			$node->nonBlankChildNodes;
 		if ( @nodes > 1 and grep { !$_->isa("XML::LibXML::Element") }
 			     @nodes ) {
-			$extra .= ">(mixed content)";
+			$extra .= ">(mixed context)";
 		}
 		elsif (@nodes and $nodes[0]->isa("XML::LibXML::Element")) {
 			$extra .= "><!-- ".@nodes
