@@ -36,12 +36,6 @@ has 'nodeName_attr' =>
 	predicate => "has_nodeName_attr",
 	;
 
-has 'xmlns_attr' =>
-	is => "rw",
-	isa => "Str",
-	predicate => "has_xmlns_attr",
-	;
-
 has 'attrName' =>
 	is => "ro",
 	isa => "Str",
@@ -56,59 +50,30 @@ has 'contents' =>
 
 method node_ok( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
 	return unless $node->nodeType == XML_ELEMENT_NODE;
-	my $got_xmlns;
-
-	if ( $self->has_xmlns or
-		     ($node->prefix||"") ne ($ctx->prefix||"") ) {
-		my $prefix = $node->prefix//"";
-		$got_xmlns = $ctx->xsi->{$prefix};
-		if ( !defined $got_xmlns ) {
-			$got_xmlns = $node->getAttribute("xmlns".(length $prefix?":$prefix":""));
-		}
+	if ( ($node->prefix||"") ne ($ctx->prefix||"") ) {
+		my $got_xmlns = ($ctx->xsi->{$node->prefix||""}||"");
 		my $wanted_xmlns = ($self->xmlns||"");
-		if ( $got_xmlns and $wanted_xmlns ne "*" and
+		if ( $wanted_xmlns ne "*" and
 			     $got_xmlns ne $wanted_xmlns ) {
-			return;
+			$ctx->exception("invalid XML namespace", $node, 1);
 		}
 	}
-	my ($ret_nodeName, $ret_xmlns) = ("", "");
-	my $wanted_nodeName = $self->nodeName;
-	if ( $wanted_nodeName ne "*" and $wanted_nodeName ne $node->localname ) {
+	# this is bad for processContents=skip + namespace="##other"
+	my $ret_nodeName = $self->nodeName eq "*" ?
+		$node->localname : "";
+	if ( !$ret_nodeName and $node->localname ne $self->nodeName ) {
 		return;
 	}
-	if ( $self->has_nodeName_attr ) {
-		$ret_nodeName = $node->localname;
-	}
-	if ( $self->has_xmlns_attr ) {
-		$ret_xmlns = $got_xmlns;
-	}
-	if ( wantarray ) {
-		return ($ret_nodeName, $ret_xmlns);
-	}
 	else {
 		return $ret_nodeName;
 	}
 }
 
 method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
-	my ($ret_nodeName, $xmlns) = $self->node_ok($node, $ctx);
-	if ( !defined $ret_nodeName ) {
-
-		# ok, not right, so figure out what we did want, in
-		# the context of the incoming document.
-		my $wanted_xmlns = ($self->xmlns||"");
-		my $wanted_prefix = $ctx->get_prefix($wanted_xmlns);
-
-		my $nodeName = $self->nodeName;
-		if ( $wanted_prefix ne "" ) {
-			$nodeName = "$wanted_prefix:$nodeName";
-			if ( $ctx->prefix_new($wanted_prefix) ) {
-				$nodeName .= " xmlns:$wanted_prefix="
-					."\"$wanted_xmlns\"";
-			}
-		}
+	my $ret_nodeName;
+	if ( !defined ($ret_nodeName = $self->node_ok($node, $ctx)) ) {
 		$ctx->exception(
-		"invalid element; expected '$nodeName'",
+		"invalid element; expected '".$self->nodeName."'",
 			$node, 1,
 		       );
 	}
@@ -116,13 +81,17 @@ method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
 	if ( $self->has_nodeClass ) {
 		# general nested XML support
 		my $marshaller = $ctx->base->get($self->nodeClass);
+		my $new_ctx = $ctx->next_ctx(
+			$node->namespaceURI,
+			$node->localname,
+		       );
 		my $value = ( $marshaller ? $marshaller->marshall_in_element(
 			$node,
-			$ctx,
+			$new_ctx,
 		       )
 				      : $node );
 		$ctx->element_ok(1);
-		return ($self->attrName => $value, $ret_nodeName, $xmlns);
+		return ($self->attrName => $value, $ret_nodeName);
 	}
 	else {
 		# XML data types
@@ -135,11 +104,7 @@ method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
 		}
 		if ( $self->has_contents ) {
 			# simple types, eg Int, Str
-			my (@childNodes) = grep {
-                		!($_->isa("XML::LibXML::Comment") or
-                        		$_->isa("XML::LibXML::Text") and $_->data =~ /\A\s+\Z/)
- 			} $node->childNodes;
-
+			my (@childNodes) = $node->nonBlankChildNodes;
 			if ( @childNodes > 1 ) {
 				# we could maybe merge CDATA nodes...
 				$ctx->exception(
@@ -157,7 +122,7 @@ method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
 				       );
 			}
 			$ctx->element_ok(1);
-			return ($self->attrName => $value, $ret_nodeName, $xmlns);
+			return ($self->attrName => $value, $ret_nodeName);
 		}
 		else {
 			# boolean
@@ -168,7 +133,7 @@ method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
 	       				);
 			}
 			$ctx->element_ok(1);
-			return ($self->attrName => 1, $ret_nodeName, $xmlns);
+			return ($self->attrName => 1, $ret_nodeName);
 		}
 	}
 }
@@ -197,7 +162,7 @@ method expected( PRANG::Graph::Context $ctx ) {
 		.">";
 }
 
-method output ( Object $item, XML::LibXML::Element $node, PRANG::Graph::Context $ctx, Item :$value, Int :$slot, Str :$name, Str :$xmlns ) {
+method output ( Object $item, XML::LibXML::Element $node, PRANG::Graph::Context $ctx, Item :$value, Int :$slot, Str :$name ) {
 	$value //= do {
 		my $accessor = $self->attrName;
 		$item->$accessor;
@@ -214,27 +179,21 @@ method output ( Object $item, XML::LibXML::Element $node, PRANG::Graph::Context
 			$self->nodeName;
 		}
 	};
-	$xmlns //= do {
-		if ( $self->has_xmlns_attr ) {
-			my $attr = $self->xmlns_attr;
-			$item->$attr;
-		}
-		else {
-			$self->xmlns // "";
-		}
-	};
 	if ( ref $name ) {
 		$name = $name->[$slot];
 	}
-	if ( ref $xmlns ) {
-		$xmlns = $xmlns->[$slot];
-	}
 
 	my $nn;
 	my $doc = $node->ownerDocument;
 	my $newctx;
 	if ( length $name ) {
-		my ($prefix, $new_prefix);
+		my ($xmlns, $prefix, $new_prefix);
+		if ( $self->has_xmlns ) {
+			$xmlns = $self->xmlns;
+			if ( $xmlns eq "*" ) {
+				$xmlns = $value->xmlns;
+			}
+		}
 		$ctx = $ctx->next_ctx( $xmlns, $name, $value );
 		$prefix = $ctx->prefix;
 		my $new_nodeName = ($prefix ? "$prefix:" : "") . $name;
@@ -248,57 +207,26 @@ method output ( Object $item, XML::LibXML::Element $node, PRANG::Graph::Context
 		$node->appendChild($nn);
 		# now proceed with contents...
 		if ( my $class = $self->nodeClass ) {
-			my $m;
-			if ( !defined $value ) {
-				$ctx->exception("required element not set");
-			}
-			elsif ( eval { $value->isa($class) }) {
-				$m = $ctx->base->get($class);
-			}
-			elsif ( eval{$value->isa("XML::LibXML::Element")} ) {
-				if ( $value->localname eq $nn->localname and
-				     ($value->namespaceURI||"") eq
-				     ($xmlns||"")
-				     ) {
-					my $nn2 = $value->cloneNode(1);
-					$node->appendChild($nn2);
-					$node->removeChild($nn);
-				}
-				else {
-					# it's just not safe to set
-					# the nodeName after the fact,
-					# so copy the children across.
-					for my $att ( $value->attributes ) {
-						next if $att->isa("XML::LibXML::Namespace");
-						$nn->setAttribute(
-							$att->localname,
-							$att->value,
-							);
-					}
-					for my $child ( $value->childNodes ) {
-						my $nn2 = $child->cloneNode(1);
-						$nn->appendChild($nn2);
-					}
-				}
-				$m = "ok";
-			}
-			elsif ( blessed $value ) {
-				# this actually indicates a type
-				# error.  currently it is required for
-				# the Whatever mapping.
+			my $m = $ctx->base->get($class);
+			if ( !$m and blessed $value ) {
 				$m = PRANG::Marshaller->get(ref $value);
 			}
-
-			if ( $m and blessed $m ) {
-				$ctx->exception("tried to serialize unblessed value $value")
-					if !blessed $value;
-				$m->to_libxml($value, $nn, $ctx);
-			}
-			elsif ( $m ) {
-				# allow value-based code above to drop through
+			if ( !$m and $value->isa("XML::LibXML::Element") ) {
+				for my $att ( $value->attributes ) {
+					$nn->setAttribute(
+						$att->localname,
+						$att->value,
+					       );
+				}
+				for my $child ( $value->childNodes ) {
+					my $nn2 = $child->cloneNode;
+					$nn->appendChild($nn2);
+				}
 			}
 			else {
-				$ctx->exception("no marshaller for '$value'");
+				$ctx->exception("tried to serialize unblessed reference")
+					if !blessed $value;
+				$m->to_libxml($value, $nn, $ctx);
 			}
 		}
 		elsif ( $self->has_contents and defined $value ) {
