@@ -5,9 +5,8 @@ use Moose;
 use MooseX::Method::Signatures;
 use Moose::Util::TypeConstraints;
 
-use XML::LibXML 1.65;
+use XML::LibXML 1.70;
 use PRANG::Util qw(types_of);
-use Carp;
 
 BEGIN {
 	class_type 'Moose::Meta::Class';
@@ -22,14 +21,6 @@ has 'class' =>
 	is => "ro",
 	required => 1,
 	handles => [qw(marshall_in_element to_libxml)],
-	trigger => sub {
-		my $self = shift;
-		my $class = $self->class;
-		if ( !$class->can("marshall_in_element") ) {
-			$class = $class->name if ref $class;
-			die "Can't marshall $class; didn't 'use PRANG::Graph' ?";
-		}
-	},
 	;
 
 our %marshallers;  # could use MooseX::NaturalKey?
@@ -49,7 +40,7 @@ method get($inv: Str $class) {
 	};
 	my $meta = $class->meta;
 	if ( $meta->does_role("PRANG::Graph") or
-		     $meta->meta->does_role("PRANG::Graph::Meta::Class")
+		     $meta->does_role("PRANG::Graph::Class")
 		    ) {
 		$marshallers{$class} ||= do {
 			$inv->new( class => $class->meta );
@@ -61,14 +52,10 @@ method get($inv: Str $class) {
 	}
 }
 
-method parse( Str :$xml, Str :$filename, GlobRef :$fh ) {
+method parse( Str $xml ) {
 
-	my $parser = XML::LibXML->new;
-	my $dom = (
-		defined $xml ? $parser->parse_string($xml) :
-		defined $filename ? $parser->parse_file($filename) :
-		defined $fh ? $parser->parse_fh($fh) :
-			croak("no input passed to parse")
+	my $dom = XML::LibXML->load_xml(
+		string => $xml,
 	       );
 
 	my $rootNode = $dom->documentElement;
@@ -111,9 +98,9 @@ method parse( Str :$xml, Str :$filename, GlobRef :$fh ) {
 
 	my $context = PRANG::Graph::Context->new(
 		base => $self,
-		xpath => "",
+		xpath => "/".$rootNode->nodeName,
 		xsi => $xsi,
-		prefix => "",
+		prefix => ($rootNode->prefix||""),
 	       );
 
 	my $rv = $self->class->marshall_in_element(
@@ -211,9 +198,9 @@ method to_xml_doc( PRANG::Graph $item ) {
 	$doc;
 }
 
-method to_xml( PRANG::Graph $item, Int $format = 0 ) {
+method to_xml( PRANG::Graph $item ) {
 	my $document = $self->to_xml_doc($item);
-	$document->toString($format);
+	$document->toString;
 }
 
 1;
